from pathlib import Path
from .files import FileOps
from .utils import divider
from .utils import Logger

class ResourcePurger:
    """Delete template artefacts not relevant to the selected language."""

    def __init__(self, fops: FileOps, logger: Logger) -> None:
        self._f = fops
        self._log = logger

    def purge(self, language: str, project_dir: Path) -> None:
        language = language.lower()
        if language == "cpp":
            self._purge_cpp(project_dir)
        elif language == "go":
            self._purge_go(project_dir)
        else:
            self._log.warn(f"Unrecognised language '{language}'; skipping purge.")
            return

        divider("Project tree after purge...")
        self._f.print_tree(project_dir)

    # ------------------------------------------------------------------ #
    def _purge_cpp(self, root: Path) -> None:
        self._f.remove_dir(root / "src" / "go")
        for f in ("Dockerfile", "Dockerfile.go", "Makefile.go"):
            self._f.remove_file(root / f)

    def _purge_go(self, root: Path) -> None:
        self._f.remove_dir(root / "src")
        self._f.remove_dir(root / "tests")
        for f in ("Dockerfile.cpp", "Makefile.cpp"):
            self._f.remove_file(root / f)



from .ascii_art import (
    goLang, divider_xl, performance_mode, divider_l, tools, divider_s,
    gRPC, divider_mono, protoC, autoMaxProcs, ants, zeroLog,
    goFast, gRpc_ProtoBuf, server, by, wjb_dev, go,
)
from .frame import TextFramer

def print_go_performance_mode_art():

    logo = TextFramer(border_char_x="", border_char_y="", padding=0, align="left")
    print(logo.frame([go]))
    
    go_performance_mode = [
        goLang, divider_xl, performance_mode, divider_l, tools, divider_s,
        gRPC, divider_mono, protoC, divider_mono, autoMaxProcs, divider_mono,
        ants, divider_mono, zeroLog,
    ]
    
    fancy = TextFramer(border_char_x="=", border_char_y="||", padding=2, align="left")
    print(fancy.frame(go_performance_mode))

    go_fast = [
        goFast, gRpc_ProtoBuf, server,
        by, wjb_dev
    ]
    footer = TextFramer(border_char_x="", border_char_y="", padding=2, align="left")
    print(footer.frame(go_fast))



from .utils.config      import PostGenConfig
from .utils             import divider
from .utils             import Logger, get_label
from .command           import CommandRunner
from .files             import FileOps
from .purge             import ResourcePurger
from .gitops            import GitOps
from .gen_ascii         import gen

def main(cfg: PostGenConfig) -> None:

    logger = init_logger(cfg.language)
    cmd = CommandRunner(logger)
    fops  = FileOps(logger)
    purge = ResourcePurger(fops, logger)
    git   = GitOps(cmd, logger)

    divider("1️⃣/ 4️⃣–  Purge template junk")
    purge.purge(cfg.language, cfg.project_dir)

    divider("2️⃣/ 4️⃣–  Initialise Git repo")
    git.init_repo(cfg.project_dir)

    divider("3️⃣/ 4️⃣–  Commit scaffold")
    git.stage_commit(cfg.project_dir)

    divider("4️⃣/ 4️⃣–  Create GitHub repo & push")
    git.push_to_github(cfg.project_dir, cfg.author, cfg.project_slug, cfg.description)

    divider("Project generation complete 🎉")
    if cfg.language == "go" and not cfg.swagger:
        gen.print_go_performance_mode_art()

def init_logger(lang) -> Logger:
    label = get_label(lang)
    return Logger(label)



import sys
from typing import TextIO

class Logger:
    def __init__(self, label: str):
        self.label = label

    def info(self, msg: str) -> None:
        print(f"{self.label} INFO: {msg}")

    def warn(self, msg: str, file: TextIO = sys.stderr) -> None:
        print(f"{self.label} WARNING: {msg}", file=file)

    def error(self, msg: str, file: TextIO = sys.stderr) -> None:
        print(f"{self.label} ERROR: {msg}", file=file)


def get_label(language: str) -> str:
    if language == "go":
        return f"[🔥Go Fast: post_gen]"
    return f"[🔥post_gen ({language})]"



from dataclasses import dataclass
from pathlib import Path

@dataclass(frozen=True, slots=True)
class PostGenConfig:
    language: str
    project_slug: str
    author: str
    description: str
    project_dir: Path
    swagger: bool
